#!/bin/bash -e
#
# simple-sync-phone:bin/simple-sync-phone-gui.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/rsync-to-phone-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/gui/rsync-to-phone-dialog-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/gui/rsync-to-phone-terminal-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/gui/rsync-to-phone-config-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/rsync-to-phone-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/gui/rsync-to-phone-ui-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/simplesshd-api.sh" || exit $?

declare -g __port_pipe__=

function __clean_pipes__ {
  if [ -n "${__port_pipe__}" ] ; then
    rm "${__port_pipe__}"
    declare -g __port_pipe__=
  fi
}

function __on_exit__ {
  echodebug 'Cleaning up...'

  __clean_pipes__
}
trap __on_exit__ EXIT

function __do_copy_common__ {
  local -r source="$1"

  local destination=
  destination="$( dialog_input 'Destination on phone' '/sdcard/Documents' )" || return $?

  rsync_to_phone_ui "${ip}" "${port}" "${copy_into}" "${source}" "${destination}"
}

function do_copy_file {
  local source=
  source="$( dialog_getopenfilename )" || return $?

  __do_copy_common__ "${source}" || return $?
}

function do_copy_directory {
  local source=
  source="$( dialog_getexistingdirectory )" || return $?

  __do_copy_common__ "${source}" || return $?
}

function do_init {
  requirements_config_init_pub_key_file
  init_authorized_keys "${PUB_KEY_FILE}" "${ip}" "${port}"
}

function get_ipv4_for_port {
  local -ri port="${port}"

  __port_pipe__="$( mktemp --dry-run --suffix='__port_pipe__.pipe' )" || return $? ## GUI FIXME
  mkfifo "${__port_pipe__}" || return $? ## GUI FIXME
  echodebug "__port_pipe__='${__port_pipe__}'" # Trace

  cat "${__port_pipe__}" &

  local -a cmd=()
  cmd+=("${PROJECT_HOME}/bin/find-open-port-on-network.sh")
  cmd+=(--port "${port}")
  cmd+=(--pipe "${__port_pipe__}")
  cmd+=(--sleep-delay 5) # TODO: add config

  if [ "${VERBOSE:-}" = 'true' ] ; then
    cmd+=(--debug)
  fi

  run_from_terminal "${cmd[@]}" || return $?

  __clean_pipes__
}

function get_ipv4 {
  local ip=
  ip="$( dialog_input 'Ip to use' '192.168.1.1' )" || return $?

  echo "${ip}"
}

function get_port {
  local port=
  local last_value='2222'

  while [ -z "${port}" ] ; do
    last_value="$( dialog_input 'Port to use' "${last_value}" )" || return $?
    if ! is_positive_integer "${last_value}" ; then
      dialog_error '--port must be a positive number.'
    elif [ "${last_value}" -lt 1 ] || [ "${last_value}" -gt 65534 ] ; then
      dialog_error '--port must be a number from 1024 to 65534'
    else
      port="${last_value}"
    fi
  done

  echo "${port}"
}

function set_user {
  local user=
  user="$( dialog_input 'username to use' "${RSYNC_TO_PHONE_REMOTE_USER}" )" || return $?

  declare -gx RSYNC_TO_PHONE_REMOTE_USER="${user}"
}

function main {
  case "${DEBUG:-}" in
    '1'|'on'|'true')
      declare -gx MESSAGE_API_SHOW_DEBUG='true'
      declare -gx VERBOSE='true'
      declare -gx __DEBUG_NMAP__='true'
      ;;
  esac

  local __GUI_OUTPUT__=
  __GUI_OUTPUT__="$( mktemp )"

  {
    cat <<EOF
simple-sync-phone : check dependencies

EOF
    requirements_tools

    cat <<EOF

---------------

EOF
    requirements_config
  } >"${__GUI_OUTPUT__}"
  dialog_textbox # Take input from "${__GUI_OUTPUT__}"

  if [ "${__REQUIREMENTS_CONFIG_SSH_KEY_FILES_COUNT__}" -eq 0 ] ; then
    requirements_config_fix || return $?
  fi

  local action=
  action="$(
    local -ra options=(
      init0         'Add public key on phone (by port)'
      init1         'Add public key on phone (by ip and port)'
      init2         'Add public key on phone (by ip and port and custom username)'
      copyf0        'Copy file to phone (by port)'
      copyf1        'Copy file to phone (by ip and port)'
      copyf2        'Copy file to phone (by ip and port and custom username)'
      copyd0        'Copy directory to phone (by port)'
      copyd1        'Copy directory to phone (by ip and port)'
      copyd2        'Copy directory to phone (by ip and port and custom username)'
      copyintod0    'Copy into directory to phone (by port)'
      copyintod1    'Copy into directory to phone (by ip and port)'
      copyintod2    'Copy into directory to phone (by ip and port and custom username)'
    )
    dialog_menu 'Choose action:' "${options[@]}"
  )"
  if [ -z "${action}" ] ; then
    dialog_error "Exit."
    return 0
  fi

  echo "action=${action}"

  local ip=
  case "${action}" in
    *'0')
      ;;

    *'1'|*'2')
      ip="$( get_ipv4 )" || return $?
      ;;

    *)
      echo "${FUNCNAME[0]}() - Wrong action='${action}' (l:${LINENO})"
      ;;
  esac

  local port=
  port="$( get_port )" || return $?

  if [ -z "${ip}" ] ; then
    ip="$( get_ipv4_for_port "${port}" )" || return $?

    echo "${FUNCNAME[0]}() - Detected ip='${ip}'"

    if [ -z "${ip}" ] ; then
      ip="$( get_ipv4 )" || return $?

      echo "${FUNCNAME[0]}() - Given ip='${ip}'"
    fi
  fi

  case "${action}" in
    *'0'|*'1')
      ;;
    *'2')
      set_user || return $?
      ;;
    *)
      echo "${FUNCNAME[0]}() - Wrong action='${action}' (l:${LINENO})"
      return 100
      ;;
  esac

  local copy_into=
  case "${action}" in
    'init'*)
      do_init || return $?
      ;;

    'copyf'*)
      copy_into='false'
      do_copy_file || return $?
      ;;

    'copyd'*)
      copy_into='false'
      do_copy_directory || return $?
      ;;

    'copyintod'*)
      copy_into='true'
      do_copy_directory || return $?
      ;;

    *)
      echo "${FUNCNAME[0]}() - Wrong action='${action}' (l:${LINENO})"
      return 100
      ;;
  esac
}

main "$@" || exit $?
